#!/usr/bin/env python
# 
# Script for segmenting cardiac images using multi-atlas label fusion.
# Note that these parameters may need to be adjusted for specific applications.

import os

# Target image
target_image = 'target_image.nii.gz'

# Atlas images and label maps that are already warped or registered to the target image space.
# Image registration can be performed using either IRTK or other registration tools (SyN, ITK etc).
images_warped = ''
labels_warped = ''
n_atlas = 5

for i in range(0, n_atlas):
    # Warped atlas image
    images_warped += 'multi_atlas/atlas_image_{0}.nii.gz '.format(i)

    # Warped atlas label map
    labels_warped += 'multi_atlas/atlas_label_{0}.nii.gz '.format(i)

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # Try majority vote
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
seg = 'fusion_mv.nii.gz'
os.system('time ../label_fusion {0} {1} {2} {3} {4} -method MV'.format(target_image, n_atlas, images_warped, labels_warped, seg))

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # Try patch-based segmentation
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Parameter file
# You may need to tune the parameters here.
par_file = 'pb_par.txt'
f = open(par_file, 'w')
f.write('Search volume size in X (pixel)   = 5\n')
f.write('Search volume size in Y (pixel)   = 5\n')
f.write('Search volume size in Z (pixel)   = 1\n')
f.write('Patch size in X (pixel)           = 7\n')
f.write('Patch size in Y (pixel)           = 7\n')
f.write('Patch size in Z (pixel)           = 1\n')
f.write('h                                 = 25000\n')
f.write('Use distance weight               = True\n')
f.close()

seg = 'fusion_pb.nii.gz'
os.system('time ../label_fusion {0} {1} {2} {3} {4} -method PB -par {5}'.format(target_image, n_atlas, images_warped, labels_warped, seg, par_file))

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # Try patch-based segmentation with augmented features
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Parameter file
# You may need to tune the parameters here.
par_file = 'pbaf_par.txt'
f = open(par_file, 'w')
f.write('Search volume size in X (pixel)   = 5\n')
f.write('Search volume size in Y (pixel)   = 5\n')
f.write('Search volume size in Z (pixel)   = 1\n')
f.write('Patch size in X (pixel)           = 7\n')
f.write('Patch size in Y (pixel)           = 7\n')
f.write('Patch size in Z (pixel)           = 1\n')
f.write('Use spatial feature               = False\n')
f.write('Use gradient feature              = True\n')
f.write('Use context feature               = True\n')
f.write('Use binary context feature        = True\n')
f.write('K                                 = 100\n') # Number of pre-selected patches
f.write('h                                 = 16\n')  # Gaussian kernel parameter
f.write('Starting radius                   = 4\n')
f.close()

seg = 'fusion_pbaf.nii.gz'
os.system('time ../label_fusion {0} {1} {2} {3} {4} -method PBAF -par {5}'.format(target_image, n_atlas, images_warped, labels_warped, seg, par_file))

# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# # # Try SVM segmentation with augmented features
# # # # # # # # # # # # # # # # # # # # # # # # # # # # # #
# Parameter file
# You may need to tune the parameters here.
par_file = 'svmaf_par.txt'
f = open(par_file, 'w')
f.write('Search volume size in X (pixel)   = 5\n')
f.write('Search volume size in Y (pixel)   = 5\n')
f.write('Search volume size in Z (pixel)   = 1\n')
f.write('Patch size in X (pixel)           = 7\n')
f.write('Patch size in Y (pixel)           = 7\n')
f.write('Patch size in Z (pixel)           = 1\n')
f.write('SVM spacing in X (pixel)          = 3\n')
f.write('SVM spacing in Y (pixel)          = 3\n')
f.write('SVM spacing in Z (pixel)          = 1\n')
f.write('Use spatial feature               = False\n')
f.write('Use gradient feature              = True\n')
f.write('Use context feature               = True\n')
f.write('Use binary context feature        = True\n')
f.write('K                                 = 100\n')        # Number of pre-selected patches
f.write('gamma                             = 0.00390625\n') # Parameters gamma and C for SVM classifier
f.write('C                                 = 0.5\n')
f.write('Starting radius                   = 4\n')
f.close()

seg = 'fusion_svmaf.nii.gz'
os.system('time ../label_fusion {0} {1} {2} {3} {4} -method SVMAF -par {5}'.format(target_image, n_atlas, images_warped, labels_warped, seg, par_file))
